import objectAssign from 'object-assign';
import ajax from './utils/ajax';
import { unix, blobToBuffer, assertOptions, getContentMd5, getSignature } from './utils';

var TinyOSS =
/*#__PURE__*/
function () {
  function TinyOSS(options) {
    if (options === void 0) {
      options = {};
    }

    assertOptions(options);
    this.opts = objectAssign({
      region: 'oss-cn-hangzhou',
      internal: false,
      cname: false,
      secure: false,
      timeout: 60000
    }, options);
    var _this$opts = this.opts,
        bucket = _this$opts.bucket,
        region = _this$opts.region,
        endpoint = _this$opts.endpoint,
        internal = _this$opts.internal;
    this.host = '';

    if (endpoint) {
      this.host = endpoint;
    } else {
      var host = bucket;

      if (internal) {
        host += '-internal';
      }

      host += "." + region + ".aliyuncs.com";
      this.host = host;
    }
  }

  var _proto = TinyOSS.prototype;

  _proto.put = function put(objectName, blob) {
    var _this = this;

    return new Promise(function (resolve, reject) {
      blobToBuffer(blob).then(function (buf) {
        var verb = 'PUT';
        var contentMd5 = getContentMd5(buf);
        var contentType = blob.type;
        var headers = {
          'Content-Md5': contentMd5,
          'Content-Type': contentType,
          'x-oss-date': new Date().toGMTString()
        };
        var _this$opts2 = _this.opts,
            bucket = _this$opts2.bucket,
            accessKeyId = _this$opts2.accessKeyId,
            accessKeySecret = _this$opts2.accessKeySecret;
        var signature = getSignature({
          verb: verb,
          contentMd5: contentMd5,
          headers: headers,
          bucket: bucket,
          objectName: objectName,
          accessKeyId: accessKeyId,
          accessKeySecret: accessKeySecret
        });
        headers.Authorization = "OSS " + accessKeyId + ":" + signature;
        var protocol = _this.opts.secure ? 'https' : 'http';
        var url = protocol + "://" + _this.host + "/" + objectName;
        return ajax(url, {
          method: verb,
          headers: headers,
          data: blob,
          timeout: _this.opts.timeout
        });
      }).then(resolve)["catch"](reject);
    });
  };

  _proto.signatureUrl = function signatureUrl(objectName, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$expires = _ref.expires,
        expires = _ref$expires === void 0 ? 3600 : _ref$expires;

    var _this$opts3 = this.opts,
        accessKeyId = _this$opts3.accessKeyId,
        accessKeySecret = _this$opts3.accessKeySecret,
        bucket = _this$opts3.bucket;
    var expireUnix = unix() + expires;
    var signature = getSignature({
      type: 'url',
      verb: 'GET',
      accessKeyId: accessKeyId,
      accessKeySecret: accessKeySecret,
      bucket: bucket,
      objectName: objectName,
      expires: expireUnix
    });
    var protocol = this.opts.secure ? 'https' : 'http';
    var url = protocol + "://" + this.host + "/" + objectName;
    url += "?OSSAccessKeyId=" + accessKeyId;
    url += "&Expires=" + expireUnix;
    url += "&Signature=" + encodeURIComponent(signature);
    return url;
  };

  return TinyOSS;
}();

export { TinyOSS as default };